// Copyright 2004 Adam Megacz, see the COPYING file for licensing [GPL]
package org.ibex;

import java.net.*;
import java.io.*;
import org.ibex.js.*;
import org.ibex.util.*;

/** Entry point for the Ibex Engine; handles splash screen, initial xwar loading, and argument processing */
public class Main {

    /**
     *  FEATURE: this should be implemented using self-emulation
     *  Used for security checks. If this is null, it means that only
     *  scripts originating from the local filesystem are loaded in
     *  the engine (maximum permissions). If scripts have been loaded
     *  from multiple locations, this will be 0.0.0.0 (the invalid
     *  IP).
     */
    public static java.net.InetAddress originAddr = null;
    public static String originHost = null;
    public static String origin = null;
    public static String initialTemplate = null;
    
    public static final Stream builtin = new Stream.Zip(new Stream.Builtin());
    public static Picture scarImage = null;

    public static void printUsage() {
        System.err.println("Usage: ibex [-s] [-v] [-l <port>/<url>] source-location [initial-template]");
        System.err.println("");
        System.err.println("Options:");
        System.err.println("    -v verbose logging (required for logging on Win32)");
        System.err.println("    -s [not yet supported]");
        System.err.println("    -l [not yet supported]");
        System.err.println("");
        System.err.println("Source-location is one of the following:");
        System.err.println("    - the path to an xwar file");
        System.err.println("    - the path to a directory to be used as an xwar");
        System.err.println("    - the http url of an xwar");
        System.err.println("");
        System.err.println("Initial-template is the path of the template to load; defaults to 'main'");
        Runtime.getRuntime().exit(-1);
    }

    public static void main(String[] args) throws Exception {
        int startargs = 0;
        while (true) {
            if (startargs > args.length - 1) printUsage();
            else if (args[startargs].equals("-v")) Log.verbose = true;
            else break;
            startargs++;
        }

        Platform.forceLoad();
        if (Log.on) for(int i=0; i<args.length; i++) Log.info(Main.class, "argument " + i + ": " + args[i]);

        initialTemplate = args.length > startargs + 1 ? args[startargs + 1] : "main";
        origin = args[startargs];

        Stream rr;
        final String startupTemplate;
        if (origin.startsWith("http://") || origin.startsWith("https://")) {
            originHost = origin.substring(origin.indexOf('/') + 2);
            originHost = originHost.substring(0, originHost.indexOf('/') == -1 ? originHost.length() : originHost.indexOf('/'));
            if (originHost.indexOf('@') != -1) originHost = originHost.substring(originHost.indexOf('@') + 1);
            originAddr = InetAddress.getByName(originHost);
            rr = builtin;
            startupTemplate = "org.ibex.builtin.splash";
        } else {
            rr = new Stream.File(origin);
            if (!new File(origin).isDirectory()) rr = new Stream.Zip(rr);
            startupTemplate = initialTemplate;
        }

        if (Log.on) Log.info(Main.class, "loading xwar");
        final Ibex ibex = new Ibex(rr);
        final JS final_rr = (JS)ibex.get("");

        scarImage =
            Picture.load((Stream)Main.builtin.get("org/ibex/builtin/scar.png"),
                         new Scheduler.Task() { public void perform() throws Exception {
                             ibex.resolveString(startupTemplate, false).call(new Box(), null, null, null, 1);
                         } });

        Scheduler.init();
    }
}
