// Copyright 2004 Adam Megacz, see the COPYING file for licensing [GPL ]

package org.ibex.js; 

/** The JavaScript Math object */
public class JSMath extends JS {

    public static JSMath singleton = new JSMath();

    private static final Double E       = new Double(java.lang.Math.E);
    private static final Double PI      = new Double(java.lang.Math.PI);
    private static final Double LN10    = new Double(java.lang.Math.log(10));
    private static final Double LN2     = new Double(java.lang.Math.log(2));
    private static final Double LOG10E  = new Double(1/java.lang.Math.log(10));
    private static final Double LOG2E   = new Double(1/java.lang.Math.log(2));
    private static final Double SQRT1_2 = new Double(1/java.lang.Math.sqrt(2));
    private static final Double SQRT2   = new Double(java.lang.Math.sqrt(2));

    public Object callMethod(Object method, Object a0, Object a1, Object a2, Object[] rest, int nargs) throws JSExn {
        switch(nargs) {
            case 0: {
                //#switch(method)
                case "random": return new Double(java.lang.Math.random());
                //#end
                break;
            }
            case 1: {
                //#switch(method)
                case "ceil": return new Long((long)java.lang.Math.ceil(toDouble(a0)));
                case "floor": return new Long((long)java.lang.Math.floor(toDouble(a0)));
                case "round": return new Long((long)java.lang.Math.round(toDouble(a0)));
                case "abs": return new Double(java.lang.Math.abs(toDouble(a0)));
                case "sin": return new Double(java.lang.Math.sin(toDouble(a0)));
                case "cos": return new Double(java.lang.Math.cos(toDouble(a0)));
                case "tan": return new Double(java.lang.Math.tan(toDouble(a0)));
                case "asin": return new Double(java.lang.Math.asin(toDouble(a0)));
                case "acos": return new Double(java.lang.Math.acos(toDouble(a0)));
                case "atan": return new Double(java.lang.Math.atan(toDouble(a0)));
                case "sqrt": return new Double(java.lang.Math.sqrt(toDouble(a0)));
                case "exp": return new Double(java.lang.Math.exp(toDouble(a0)));
                case "log": return new Double(java.lang.Math.log(toDouble(a0)));
                //#end
                break;
            }
            case 2: {
                //#switch(method)
                case "min": return new Double(java.lang.Math.min(toDouble(a0), toDouble(a1)));
                case "max": return new Double(java.lang.Math.max(toDouble(a0), toDouble(a1)));
                case "pow": return new Double(java.lang.Math.pow(toDouble(a0), toDouble(a1)));
                case "atan2": return new Double(java.lang.Math.atan2(toDouble(a0), toDouble(a1)));
                //#end
                break;
            }
        }
        return super.callMethod(method, a0, a1, a2, rest, nargs);
    }

    public void put(Object key, Object val) { return; }

    public Object get(Object key) throws JSExn {
        //#switch(key)
        case "E": return E;
        case "LN10": return LN10;
        case "LN2": return LN2;
        case "LOG10E": return LOG10E;
        case "LOG2E": return LOG2E;
        case "PI": return PI;
        case "SQRT1_2": return SQRT1_2;
        case "SQRT2": return SQRT2;
        case "ceil": return METHOD;
        case "floor": return METHOD;
        case "round": return METHOD;
        case "min": return METHOD;
        case "max": return METHOD;
        case "pow": return METHOD;
        case "atan2": return METHOD;
        case "abs": return METHOD;
        case "sin": return METHOD;
        case "cos": return METHOD;
        case "tan": return METHOD;
        case "asin": return METHOD;
        case "acos": return METHOD;
        case "atan": return METHOD;
        case "sqrt": return METHOD;
        case "exp": return METHOD;
        case "log": return METHOD;
        case "random": return METHOD;
        //#end
        return super.get(key);
    }
}
