package org.xwt.translators;
import org.ibex.*;
import org.ibex.util.*;
import java.io.*;

import org.xwt.mips.Runtime;

// FEATURE: use streams, not memoryfont's
// FEATURE: kerning pairs
public class Freetype {

    public Freetype() { }

    private int mem_allocated = 0;
    private Runtime vm = null;

    private Stream loadedStream = null;

    public void loadFontByteStream(Stream res) {
        try {
            Log.info(this, "loading font " + res);
            loadedStream = res;
            InputStream is = Stream.getInputStream(res);
            byte[] fontstream = InputStreamToByteArray.convert(is);
            vm = (Runtime)Class.forName("org.xwt.translators.MIPSApps").newInstance();
            int baseAddr = vm.sbrk(fontstream.length);
            vm.copyout(fontstream, baseAddr, fontstream.length);
            vm.setUserInfo(0, baseAddr);
            vm.setUserInfo(1, fontstream.length);
            vm.start(new String[]{ "freetype" });
            vm.execute();
            if(vm.getState() == Runtime.DONE) throw new Error("Freetype VM exited: " + vm.exitStatus());
        } catch (Exception e) {
            Log.info(this, e);
        }
    }

    public synchronized void renderGlyph(Font.Glyph glyph) throws IOException {
        try {
            Log.debug(this, "rasterizing glyph " + glyph.c + " of font " + glyph.font);
            if (loadedStream != glyph.font.stream) loadFontByteStream(glyph.font.stream);
            vm.setUserInfo(2, (int)glyph.c);
            vm.setUserInfo(3, (int)glyph.c);
            vm.setUserInfo(4, glyph.font.pointsize);
            long start = System.currentTimeMillis();
            vm.execute();
            glyph.font.max_ascent = vm.getUserInfo(8);
            glyph.font.max_descent = vm.getUserInfo(9);
            glyph.baseline = vm.getUserInfo(10);
            glyph.advance = vm.getUserInfo(11);
            
            glyph.width = vm.getUserInfo(6);
            glyph.height = vm.getUserInfo(7);
            
            glyph.data = new byte[glyph.width * glyph.height];
            int addr = vm.getUserInfo(5);
            vm.copyin(addr, glyph.data, glyph.width * glyph.height);
            glyph.isLoaded = true;
            
        } catch (Exception e) {
            Log.info(this, e);
        }
    }
}
